
# Slixmpp: The Slick XMPP Library
# Copyright (C) 2013 Nathanael C. Fritz, Lance J.T. Stout
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
from slixmpp.plugins.base import register_plugin

from slixmpp.plugins.xep_0096 import stanza
from slixmpp.plugins.xep_0096.stanza import File
from slixmpp.plugins.xep_0096.file_transfer import XEP_0096


register_plugin(XEP_0096)




# Slixmpp: The Slick XMPP Library
# Copyright (C) 2020 Mathieu Pasquet
# This file is part of Slixmpp.
# See the file LICENSE for copying permission.
import asyncio
import os
from unittest import IsolatedAsyncioTestCase
from typing import (
    Dict,
    List,
    Optional,
)

from slixmpp import JID
from slixmpp.clientxmpp import ClientXMPP


class SlixIntegration(IsolatedAsyncioTestCase):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.clients = []
        self.addAsyncCleanup(self._destroy)

    def envjid(self, name: str, *, default: Optional[str] = None) -> JID:
        """Get a JID from an env var"""
        value = os.getenv(name, default=default)
        return JID(value)

    def envstr(self, name):
        """get a str from an env var"""
        return os.getenv(name)

    def register_plugins(self, plugins: List[str], configs: Optional[List[Dict]] = None):
        """Register plugins on all known clients"""
        for index, plugin in enumerate(plugins):
            for client in self.clients:
                if configs is not None:
                    client.register_plugin(plugin, pconfig=configs[index])
                else:
                    client.register_plugin(plugin)

    def add_client(self, jid: JID, password: str):
        """Register a new client"""
        self.clients.append(ClientXMPP(jid, password))

    async def connect_clients(self):
        """Connect all clients"""
        for client in self.clients:
            client.connect()
        wait = [client.wait_until('session_start') for client in self.clients]
        await asyncio.gather(*wait)

    async def _destroy(self):
        """Kill all clients"""
        for client in self.clients:
            client.abort()



import asyncio
import ssl
import unittest
from slixmpp import ClientXMPP
from slixmpp.test.integration import SlixIntegration


class TestTLS(SlixIntegration):
    async def test_connect_direct_tls(self):
        """Check that we can force connection in direct TLS"""
        client = ClientXMPP(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        client.enable_direct_tls = True
        client.enable_starttls = False
        _, pending = await asyncio.wait(
            [asyncio.ensure_future(client.connect()),
             asyncio.ensure_future(client.wait_until('session_start'))],
            timeout=10
        )
        self.assertFalse(bool(pending))
        extra_info = client.transport.get_extra_info('peername')
        self.assertEqual(extra_info[1], 5223)
        self.assertTrue(isinstance(client.socket, ssl.SSLObject))

    async def test_connect_starttls(self):
        """Check that we can force connection in starttls"""
        client = ClientXMPP(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        client.enable_direct_tls = False
        client.enable_starttls = True
        _, pending = await asyncio.wait(
            [asyncio.ensure_future(client.connect()),
             asyncio.ensure_future(client.wait_until('session_start'))],
            timeout=10
        )
        self.assertFalse(bool(pending))
        extra_info = client.transport.get_extra_info('peername')
        self.assertEqual(extra_info[1], 5222)
        self.assertTrue(isinstance(client.socket, ssl.SSLObject))

    async def test_connect_custom(self):
        """Check that we can connect with custom params"""
        client = ClientXMPP(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        client.enable_direct_tls = False
        client.enable_starttls = True
        _, pending = await asyncio.wait(
            [asyncio.ensure_future(client.connect(host=self.envjid('CI_ACCOUNT1').host, port=5222)),
             asyncio.ensure_future(client.wait_until('session_start'))],
            timeout=10
        )
        self.assertFalse(bool(pending))
        extra_info = client.transport.get_extra_info('peername')
        self.assertEqual(extra_info[1], 5222)
        self.assertTrue(isinstance(client.socket, ssl.SSLObject))

    async def test_connect_custom_fail(self):
        """Check that providing bad custom params fail"""
        client = ClientXMPP(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        client.enable_direct_tls = True
        client.enable_starttls = True
        fut = asyncio.Future()
        def handler(event):
            if not fut.done():
                fut.set_result(True)

        with client.event_handler('connection_failed', handler):
            await client.connect(host=self.envjid('CI_ACCOUNT1').host, port=9999)
            res = await asyncio.wait([fut], timeout=1)
            self.assertTrue(res)

    async def test_connect_custom_fail2(self):
        """Check that providing bad custom params fail"""
        client = ClientXMPP(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )
        client.enable_direct_tls = False
        client.enable_starttls = True
        fut = asyncio.Future()
        def handler(event):
            if not fut.done():
                fut.set_result(True)

        with client.event_handler('connection_failed', handler):
            await client.connect(host=self.envjid('CI_ACCOUNT1').host, port=5223)
            res = await asyncio.wait([fut], timeout=1)
            self.assertTrue(res)

    async def test_validate_cert_custom(self):
        """Check that we can validate the cert manually"""
        client = ClientXMPP(
            self.envjid('CI_ACCOUNT1'),
            self.envstr('CI_ACCOUNT1_PASSWORD'),
        )

        def handler(cert):
            # Abort!
            client.abort()

        with client.event_handler('ssl_cert', handler):
            _, pending = await asyncio.wait(
                [asyncio.ensure_future(client.connect()),
                 asyncio.ensure_future(client.wait_until('killed'))],
                timeout=10
            )
            self.assertFalse(bool(pending))


suite = unittest.TestLoader().loadTestsFromTestCase(TestTLS)
